<?php
class Employee extends Person
{
	/*
	Determines if a given person_id is an employee
	*/
	public function exists($person_id)
	{
		$this->db->from('employees');	
		$this->db->join('people', 'people.person_id = employees.person_id');
		$this->db->where('employees.person_id', $person_id);

		return ($this->db->get()->num_rows() == 1);
	}	

	/*
	Gets total of rows
	*/
	public function get_total_rows()
	{
		$this->db->from('employees');
		$this->db->where('deleted', 0);

		return $this->db->count_all_results();
	}

	/*
	Returns all the employees
	*/
	public function get_all($limit = 10000, $offset = 0)
	{
		$this->db->from('employees');
		$this->db->where('deleted', 0);		
		$this->db->join('people', 'employees.person_id = people.person_id');			
		$this->db->order_by('last_name', 'asc');
		$this->db->limit($limit);
		$this->db->offset($offset);

		return $this->db->get();		
	}
	
	/*
	Gets information about a particular employee
	*/
	public function get_info($employee_id)
	{
		$this->db->from('employees');	
		$this->db->join('people', 'people.person_id = employees.person_id');
		$this->db->where('employees.person_id', $employee_id);
		$query = $this->db->get();

		if($query->num_rows() == 1)
		{
			return $query->row();
		}
		else
		{
			//Get empty base parent object, as $employee_id is NOT an employee
			$person_obj = parent::get_info(-1);

			//Get all the fields from employee table
			//append those fields to base parent object, we we have a complete empty object
			foreach($this->db->list_fields('employees') as $field)
			{
				$person_obj->$field = '';
			}

			return $person_obj;
		}
	}

	/*
	Gets information about multiple employees
	*/
	public function get_multiple_info($employee_ids)
	{
		$this->db->from('employees');
		$this->db->join('people', 'people.person_id = employees.person_id');		
		$this->db->where_in('employees.person_id', $employee_ids);
		$this->db->order_by('last_name', 'asc');

		return $this->db->get();		
	}

	/*
	Inserts or updates an employee
	*/
	public function save_employee(&$person_data, &$employee_data, &$grants_data, $employee_id = FALSE)
	{
		$success = FALSE;

		//Run these queries as a transaction, we want to make sure we do all or nothing
		$this->db->trans_start();

		if(parent::save($person_data, $employee_id))
		{
			if(!$employee_id || !$this->exists($employee_id))
			{
				$employee_data['person_id'] = $employee_id = $person_data['person_id'];
				$success = $this->db->insert('employees', $employee_data);
			}
			else
			{
				$this->db->where('person_id', $employee_id);
				$success = $this->db->update('employees', $employee_data);
			}

			//We have either inserted or updated a new employee, now lets set permissions. 
			if($success)
			{
				//First lets clear out any grants the employee currently has.
				$success = $this->db->delete('grants', array('person_id' => $employee_id));
				
				//Now insert the new grants
				if($success)
				{
					foreach($grants_data as $permission_id)
					{
						$success = $this->db->insert('grants', array('permission_id' => $permission_id, 'person_id' => $employee_id));
					}
				}
			}
		}

		$this->db->trans_complete();

		$success &= $this->db->trans_status();

		return $success;
	}

	/*
	Deletes one employee
	*/
	public function delete($employee_id)
	{
		$success = FALSE;

		//Don't let employees delete theirself
		if($employee_id == $this->get_logged_in_employee_info()->person_id)
		{
			return FALSE;
		}

		//Run these queries as a transaction, we want to make sure we do all or nothing
		$this->db->trans_start();

		//Delete permissions
		if($this->db->delete('grants', array('person_id' => $employee_id)))
		{	
			$this->db->where('person_id', $employee_id);
			$success = $this->db->update('employees', array('deleted' => 1));
		}

		$this->db->trans_complete();

		return $success;
	}

	/*
	Deletes a list of employees
	*/
	public function delete_list($employee_ids)
	{
		$success = FALSE;

		//Don't let employees delete theirself
		if(in_array($this->get_logged_in_employee_info()->person_id, $employee_ids))
		{
			return FALSE;
		}

		//Run these queries as a transaction, we want to make sure we do all or nothing
		$this->db->trans_start();

		$this->db->where_in('person_id', $employee_ids);
		//Delete permissions
		if($this->db->delete('grants'))
		{
			//delete from employee table
			$this->db->where_in('person_id', $employee_ids);
			$success = $this->db->update('employees', array('deleted' => 1));
		}

		$this->db->trans_complete();

		return $success;
 	}

	/*
	Get search suggestions to find employees
	*/
	public function get_search_suggestions($search, $limit = 5)
	{
		$suggestions = array();

		$this->db->from('employees');
		$this->db->join('people', 'employees.person_id = people.person_id');
		$this->db->group_start();
			$this->db->like('first_name', $search);
			$this->db->or_like('last_name', $search); 
			$this->db->or_like('CONCAT(first_name, " ", last_name)', $search);
		$this->db->group_end();
		$this->db->where('deleted', 0);
		$this->db->order_by('last_name', 'asc');
		foreach($this->db->get()->result() as $row)
		{
			$suggestions[] = array('value' => $row->person_id, 'label' => $row->first_name.' '.$row->last_name);
		}

		$this->db->from('employees');
		$this->db->join('people', 'employees.person_id = people.person_id');
		$this->db->where('deleted', 0);
		$this->db->like('email', $search);
		$this->db->order_by('email', 'asc');
		foreach($this->db->get()->result() as $row)
		{
			$suggestions[] = array('value' => $row->person_id, 'label' => $row->email);
		}

		$this->db->from('employees');
		$this->db->join('people', 'employees.person_id = people.person_id');
		$this->db->where('deleted', 0);
		$this->db->like('username', $search);
		$this->db->order_by('username', 'asc');
		foreach($this->db->get()->result() as $row)
		{
			$suggestions[] = array('value' => $row->person_id, 'label' => $row->username);
		}

		$this->db->from('employees');
		$this->db->join('people', 'employees.person_id = people.person_id');
		$this->db->where('deleted', 0);
		$this->db->like('phone_number', $search);
		$this->db->order_by('phone_number', 'asc');
		foreach($this->db->get()->result() as $row)
		{
			$suggestions[] = array('value' => $row->person_id, 'label' => $row->phone_number);
		}

		//only return $limit suggestions
		if(count($suggestions > $limit))
		{
			$suggestions = array_slice($suggestions, 0, $limit);
		}

		return $suggestions;
	}

 	/*
	Gets rows
	*/
	public function get_found_rows($search)
	{
		$this->db->from('employees');
		$this->db->join('people', 'employees.person_id = people.person_id');
		$this->db->group_start();
			$this->db->like('first_name', $search);
			$this->db->or_like('last_name', $search);
			$this->db->or_like('email', $search);
			$this->db->or_like('phone_number', $search);
			$this->db->or_like('username', $search);
			$this->db->or_like('CONCAT(first_name, " ", last_name)', $search);
		$this->db->group_end();
		$this->db->where('deleted', 0);

		return $this->db->get()->num_rows();
	}

	/*
	Performs a search on employees
	*/
	public function search($search, $rows = 0, $limit_from = 0, $sort = 'last_name', $order = 'asc')
	{
		$this->db->from('employees');
		$this->db->join('people', 'employees.person_id = people.person_id');
		$this->db->group_start();
			$this->db->like('first_name', $search);
			$this->db->or_like('last_name', $search);
			$this->db->or_like('email', $search);
			$this->db->or_like('phone_number', $search);
			$this->db->or_like('username', $search);
			$this->db->or_like('CONCAT(first_name, " ", last_name)', $search);
		$this->db->group_end();
		$this->db->where('deleted', 0);
		$this->db->order_by($sort, $order);

		if($rows > 0)
		{
			$this->db->limit($rows, $limit_from);
		}

		return $this->db->get();	
	}

	/*
	Attempts to login employee and set session. Returns boolean based on outcome.
	*/
	public function login($username, $password)
	{
		$query = $this->db->get_where('employees', array('username' => $username, 'deleted' => 0), 1);

		if($query->num_rows() == 1)
		{
			$row = $query->row();

			// compare passwords depending on the hash version
			if ($row->hash_version == 1 && $row->password == md5($password))
			{
				$this->db->where('person_id', $row->person_id);
				$this->session->set_userdata('person_id', $row->person_id);
				$password_hash = password_hash($password, PASSWORD_DEFAULT);

				return $this->db->update('employees', array('hash_version' => 2, 'password' => $password_hash));
			}
			else if ($row->hash_version == 2 && password_verify($password, $row->password))
			{
				$this->session->set_userdata('person_id', $row->person_id);

				return TRUE;
			}

		}

		return FALSE;
	}

	/*
	Logs out a user by destorying all session data and redirect to login
	*/
	public function logout()
	{
		$this->session->sess_destroy();

		redirect('login');
	}
	
	/*
	Determins if a employee is logged in
	*/
	public function is_logged_in()
	{
		return ($this->session->userdata('person_id') != FALSE);
	}

	/*
	Gets information about the currently logged in employee.
	*/
	public function get_logged_in_employee_info()
	{
		if($this->is_logged_in())
		{
			return $this->get_info($this->session->userdata('person_id'));
		}

		return FALSE;
	}

	/*
	Determines whether the employee has access to at least one submodule
	 */
	public function has_module_grant($permission_id, $person_id)
	{
		$this->db->from('grants');
		$this->db->like('permission_id', $permission_id, 'after');
		$this->db->where('person_id', $person_id);
		$result_count = $this->db->get()->num_rows();

		if($result_count != 1)
		{
			return ($result_count != 0);
		}

		return $this->has_subpermissions($permission_id);
	}

 	/*
	Checks permissions
	*/
	public function has_subpermissions($permission_id)
	{
		$this->db->from('permissions');
		$this->db->like('permission_id', $permission_id.'_', 'after');

		return ($this->db->get()->num_rows() == 0);
	}

	/*
	Determines whether the employee specified employee has access the specific module.
	*/
	public function has_grant($permission_id, $person_id)
	{
		//if no module_id is null, allow access
		if($permission_id == null)
		{
			return TRUE;
		}

		$query = $this->db->get_where('grants', array('person_id' => $person_id, 'permission_id' => $permission_id), 1);

		return ($query->num_rows() == 1); 
	}

 	/*
	Gets employee permission grants
	*/
	public function get_employee_grants($person_id)
	{
		$this->db->from('grants');
		$this->db->where('person_id', $person_id);

		return $this->db->get()->result_array();
	}

	/* Get comision vendedor*/
	public function get_seller_percents($fecha_ini, $fecha_fin)
	{

		$sql="select q.seller_id, q.seller, sum(q.total) as total, sum(q.comision) as comision from (
			SELECT ospos_sales.seller_id,ospos_sales.sale_id,concat(ospos_people.first_name,' ',ospos_people.last_name) as seller, (ospos_sales_items.quantity_purchased * ospos_sales_items.item_unit_price) as total,
			ospos_sales_items.item_id,
			(ospos_sales_items.quantity_purchased * ospos_sales_items.item_unit_price)*
			(select l.pay_percent
			from ospos_price_list l,ospos_prices_items i
			where i.price_id=l.price_id
			and i.item_id=ospos_sales_items.item_id
			and l.quantity_purchased <= ospos_sales_items.quantity_purchased
			order by l.quantity_purchased desc
			limit 1)/100 as comision
			FROM ospos_sales,ospos_sales_items,ospos_people
			WHERE ospos_sales.sale_id=ospos_sales_items.sale_id
			and ospos_sales.seller_id=ospos_people.person_id
			and date(ospos_sales.sale_time) between date(?) and date(?)
			) q
			GROUP by q.seller_id";

		$query=$this->db->query($sql,array($fecha_ini, $fecha_fin));
		$r=$query->result();
		return $r;
	}

	/* Get comision DETALLE vendedor*/
	public function get_seller_detail($seller_id,$fecha_ini, $fecha_fin)
	{

		$sql="select sale_date,sale_id,customer,sum(total) as total, sum(comision) as comision, sum(costo) as costo from (
			select date(ospos_sales.sale_time) as sale_date,ospos_sales.customer_id,ospos_sales.sale_id,concat(ospos_people.first_name,' ',ospos_people.last_name,' ',ospos_people.address_2) as customer, (ospos_sales_items.quantity_purchased * ospos_sales_items.item_unit_price) as total,
						ospos_sales_items.item_id,
						(ospos_sales_items.quantity_purchased * ospos_sales_items.item_unit_price)*
						(select l.pay_percent
						from ospos_price_list l,ospos_prices_items i
						where i.price_id=l.price_id
						and i.item_id=ospos_sales_items.item_id
						and l.quantity_purchased <= ospos_sales_items.quantity_purchased
						order by l.quantity_purchased desc
						limit 1)/100 as comision,
    (ospos_sales_items.quantity_purchased * ospos_sales_items.item_cost_price) as costo
						FROM ospos_sales,ospos_sales_items,ospos_people
						WHERE ospos_sales.sale_id=ospos_sales_items.sale_id
						and ospos_sales.customer_id=ospos_people.person_id
			    		and ospos_sales.seller_id=?
			    		and date(ospos_sales.sale_time) between date(?) and date(?)
			    ) q
			    GROUP BY sale_id";		

		$query=$this->db->query($sql,array($seller_id,$fecha_ini, $fecha_fin));
		$r=$query->result();
		return $r;
	}


	

	/* Get comision vendedor*/
	public function get_sale_detail($sale_id)
	{

		$sql="select i.item_number,i.name, s.item_cost_price,s.item_unit_price, s.quantity_purchased,s.item_unit_price * s.quantity_purchased as total
			from ospos_sales_items s,ospos_items i
			where i.item_id=s.item_id
			and s.sale_id=?
			order by s.line";

		$query=$this->db->query($sql,array($sale_id));
		$r=$query->result();
		return $r;
	}

	/*JORNALEROS*/
	public function get_jornaleros()
	{	
		$this->db->select('ospos_employees.person_id');
		$this->db->select('ospos_people.first_name');
		$this->db->select('ospos_people.last_name');
		$this->db->select('ospos_people.address_2');
		$this->db->from('ospos_employees');
		$this->db->join('ospos_people', 'ospos_employees.person_id=ospos_people.person_id');
		$this->db->where('ospos_employees.deleted', 0);
		$this->db->join('ospos_grants', 'ospos_employees.person_id=ospos_grants.person_id');
		$this->db->where('ospos_grants.permission_id', 'sales_Jornalero');
		return $this->db->get();


		/*$sql="select 
				ospos_employees.person_id,
				ospos_people.first_name,
				ospos_people.last_name,
				ospos_people.address_2,
				from ospos_employees,ospos_people,ospos_grants
				where ospos_employees.deleted = 0
				and ospos_employees.person_id = ospos_people.person_id
				and ospos_employees.person_id = ospos_grants.person_id
				ospos_grants.permission_id = 'sales_Jornalero'";

		$query=$this->db->query($sql,array($sale_id));
		$r=$query->result();
		return $r;*/
	}

	public function get_debitos_creditos($employee_id,$fecha_ini, $fecha_fin)
	{	
		$this->db->select('ospos_employees_debit_credit.debit_credit_time');
		$this->db->select('ospos_employees_debit_credit.description');
		$this->db->select('ospos_employees_debit_credit.debit_amount');
		$this->db->select('ospos_employees_debit_credit.credit_amount');
		$this->db->from('ospos_employees_debit_credit');
		$this->db->select('ospos_employees_debit_credit.credit_amount');
		$this->db->where('ospos_employees_debit_credit.employee_id', $employee_id);
		$this->db->where('date(ospos_employees_debit_credit.debit_credit_time) >= ',$fecha_ini);
		$this->db->where('date(ospos_employees_debit_credit.debit_credit_time) <= ',$fecha_fin);
		return $this->db->get();	
			
	}
	/*FIN JORNALEROS*/

	/* Get depositos para empleados*/
	public function get_users_locations()
	{
		$admin=2;
		$sql="select e.username,p.location_id 
			from ospos_employees e,ospos_grants g,ospos_permissions p
			where e.person_id >?
			and e.deleted=0
			and g.person_id=e.person_id
			and p.permission_id=g.permission_id
			and p.module_id='sales'
			and p.location_id is not null";

		$query=$this->db->query($sql,array($admin));
		$r=$query->result();
		return $r;
	}

	public function get_minorista($employee_id)
	{
		$this->db->select('minorista');
		$this->db->from('ospos_employees');
		$this->db->where('ospos_employees.person_id', $employee_id);

		return $this->db->get()->row()->minorista;
	}

	public function get_empleados($seller_id)
	{
		$admin=2;
		$sql="SELECT p.first_name, p.last_name, p.person_id
		FROM ospos_people p, ospos_employees e
		WHERE p.person_id=e.person_id
		AND p.person_id=?
		union all
		SELECT p.first_name, p.last_name, p.person_id
		FROM ospos_people p, ospos_employees e
		WHERE p.person_id=e.person_id
		and p.person_id!=?
		AND p.person_id>?";

		$query=$this->db->query($sql,array($seller_id,$seller_id,$admin));
		$r=$query->result();
		return $r;
	}
}
?>

